# -*- coding: utf-8 -*-

from PyQt4 import QtCore, QtGui
from qgis.gui import *
from qgis.core import *

class VectorLayerSelectorItem(QtGui.QWidget):

    def __init__(self, layer, parent = None):
        super(VectorLayerSelectorItem, self).__init__(parent)

        # init color from Qgis layer proeperties
        self.color = self.getVectorLayerColor(layer)

        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setSpacing(2)
        self.horizontalLayout.setMargin(0)
        
        self.checkbox = QtGui.QCheckBox()
        self.checkbox.setText(layer.name())
        
        self.colorpicker = QtGui.QPushButton()
        self.updateColor()
        self.colorpicker.clicked.connect(self.selectColor)
        
        self.horizontalLayout.addWidget(self.checkbox)
        self.horizontalLayout.addStretch()
        self.horizontalLayout.addWidget(self.colorpicker)
        self.setLayout(self.horizontalLayout)

    def selectColor(self):
        color = QtGui.QColorDialog.getColor(self.getColor(), self, "Selectionner une couleur")
        if color.isValid():
            self.color = color
        self.updateColor()

    def updateColor(self):
        pixmap = QtGui.QPixmap(self.colorpicker.size())
        pixmap.fill(self.getColor())
        icon = QtGui.QIcon(pixmap)
        self.colorpicker.setIcon(icon)
        
    def getVectorLayerColor(self, layer):
        color = QtCore.Qt.white
        if layer.isUsingRendererV2():
            rendererV2 = layer.rendererV2()
            if isinstance(rendererV2,QgsSingleSymbolRendererV2):
                sym = rendererV2.symbol()
                color = sym.color()          
        else:
            renderer = layer.renderer()
            if isinstance(renderer,QgsSingleSymbolRenderer):
                sym = renderer.symbol()
                color = sym.fillColor()
        return color

    def setChecked(self, val):
        self.checkbox.setChecked(val)

    def isChecked(self):
        return self.checkbox.isChecked()

    def getColor(self):
        return self.color
        
    def getLabel(self):
        return self.checkbox.text()