# -*- coding: utf-8 -*-

"""
/***************************************************************************
 SupervisedClassificationDialog
                                 A QGIS plugin
 Supervised classification
                             -------------------
        begin                : 2012-07-27
        copyright            : (C) 2012 by CNES
        email                : julien.malik@c-s.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from PyQt4 import QtCore, QtGui
from RasterLayerSelectorTable import RasterLayerSelectorTable
from VectorLayerSelectorTable import VectorLayerSelectorTable
from ConfusionMatrixViewer import ConfusionMatrixViewer

from QGisLayers import QGisLayers
from QGisLayers import QGisLayerType

from OSIdentifier import OSIdentifier

import tempfile
import os
import subprocess
import shutil

# create the dialog for zoom to point
class SupervisedClassificationDialog(QtGui.QDialog):
    def __init__(self):
        QtGui.QDialog.__init__(self)
        self.setupUi()
        
    def setupUi(self):
        self.setWindowTitle(u"Classification Supervisée")
        
        self.mainlayout = QtGui.QVBoxLayout()

        rasterlayers = QGisLayers.getRasterLayers()
        self.rasterlayerselector = RasterLayerSelectorTable(rasterlayers)

        vectorlayers = QGisLayers.getVectorLayers(QGisLayerType.POLYGON)
        self.vectorlayerselector = VectorLayerSelectorTable(vectorlayers)

        self.layerlayout = QtGui.QHBoxLayout()
        self.layerlayout.addWidget(self.rasterlayerselector)
        self.layerlayout.addWidget(self.vectorlayerselector)
        
        self.outputlayout = QtGui.QHBoxLayout()
        
        self.outputdirwidget = QtGui.QLineEdit()
        self.outputdirselectorbutton = QtGui.QPushButton("...")
        self.setOutputDir( tempfile.mkdtemp(prefix='ClassificationSupervisee_', dir=None) )
        
        self.outputlayout.addWidget( QtGui.QLabel(u"Répertoire de sortie") )
        self.outputlayout.addWidget( self.outputdirwidget )
        self.outputlayout.addWidget( self.outputdirselectorbutton )
        
        self.buttonBox = QtGui.QDialogButtonBox()
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        
        self.classifButton = QtGui.QPushButton("Classification")
        self.cancelButton = QtGui.QPushButton("Annuler")
        
        self.buttonBox.addButton(self.classifButton, QtGui.QDialogButtonBox.AcceptRole)
        self.buttonBox.addButton(self.cancelButton, QtGui.QDialogButtonBox.RejectRole)

        self.mainlayout.addLayout(self.layerlayout)
        self.mainlayout.addLayout(self.outputlayout)
        self.mainlayout.addWidget(self.buttonBox)
        self.setLayout(self.mainlayout)
        
        QtCore.QObject.connect(self.classifButton, QtCore.SIGNAL("clicked()"), self.classify)
        QtCore.QObject.connect(self.cancelButton, QtCore.SIGNAL("clicked()"), self.cancelPressed)
        
        QtCore.QObject.connect(self.outputdirselectorbutton, QtCore.SIGNAL("clicked()"), self.selectOutputDir)

    def cancelPressed(self):
        self.close()

    def setOutputDir(self, dirname):
        self.outputdirwidget.setText(dirname)
        
    def getOutputDir(self):
        return str(self.outputdirwidget.text())
        
    def selectOutputDir(self):
        filedialog = QtGui.QFileDialog()
        filedialog.setConfirmOverwrite(True);
        filedialog.setFileMode(QtGui.QFileDialog.Directory);
        filedialog.setNameFilter(u"Répertoire");
        if filedialog.exec_():
            self.setOutputDir(filedialog.selectedFiles()[0])

    def classify(self):

        # Get rasters
        selectedrasterlayers = self.rasterlayerselector.getSelectedOptions()
        if len(selectedrasterlayers) < 1:
            QtGui.QMessageBox.critical( self, \
                                        u"Erreur", \
                                        u"Aucune couche raster sélectionnée" )
            return

        # Get vectors
        selectedvectorlayers = self.vectorlayerselector.getSelectedOptions()
        if len(selectedvectorlayers) < 2:
            QtGui.QMessageBox.critical( self, \
                                        u"Erreur", \
                                        u"Au minimum deux couches vecteur doivent être sélectionnées" )
            return
        
        # Build commandline
        rasterlist = ""
        for r in selectedrasterlayers:
            rasterlist += '"%s" ' % (unicode(r.source()))
        
        vectorlist = ""
        labeldescriptor = {}
        label = 0
        for v in selectedvectorlayers:
            shpfilepath = v[0].source()
            classcolor = v[1]
            classlabel = v[2]
            
            labeldescriptor[label] = (classcolor, classlabel)
            label += 1
            
            vectorlist += '"%s" ' % (unicode(shpfilepath))
            
        
        outputdir = self.getOutputDir()
        
        outputlog = os.path.join(outputdir, 'output.log')
        outputclassification = os.path.join(outputdir, 'classification.tif')
        outputvalidation = os.path.join(outputdir, 'classification.matriceconfusion.txt')
        
        if OSIdentifier.isWindows():
            launcher = "%s/win32/bin/classification.bat" % os.path.dirname(__file__)
            command = ( "%s "
                        "-io.il %s "
                        "-io.vd %s "
                        "-io.out %s uint8 "
                        "-io.val %s"
                        %  (launcher,
                            rasterlist,
                            vectorlist, 
                            outputclassification,
                            outputvalidation) )
        else:
            command = ( "ITK_AUTOLOAD_PATH=~/Projets/otb/build/ViewerEducation/lib "
                        "otbcli "
                        "-io.il %s "
                        "-io.vd %s "
                        "-io.out %s uint8 "
                        "-io.val %s"
                        %  (rasterlist,
                            vectorlist, 
                            outputclassification,
                            outputvalidation) )

        # Execute commandline
        try:
                proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stdin=subprocess.PIPE,stderr=subprocess.STDOUT, universal_newlines=False)
                
                loglines = []
                for line in iter(proc.stdout.readline, ""):
                    loglines.append(line)
                    
                proc.wait()
                if proc.returncode != 0:
                    raise OSError

        except OSError:
            mbox = QtGui.QMessageBox()
            mbox.setWindowTitle(u"Erreur")
            mbox.setText(u"Une erreur a été rencontrée lors de la classification")
            
            detailedtext = (u"Code de retour : %s%s"
                             "Commande : %s%s"
                             "Sortie standard :%s%s"
                              % ( proc.returncode, 2*os.linesep,
                                  command, 2*os.linesep,
                                  os.linesep, u''.join([u'%s%s' % (unicode(c, 'cp1252'), os.linesep) for c in loglines]) ) )

            mbox.setDetailedText(detailedtext)
            mbox.setIcon(QtGui.QMessageBox.Critical)
            mbox.setStandardButtons(QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel)
            ret = mbox.exec_()
            
            os.remove(outputlog)
            os.remove(outputclassification)
            os.remove(outputvalidation)
            
            return

        # Report results
        with open(outputvalidation, 'r') as valid:
            matrixtext = valid.read()

        layernames = []
        for v in selectedvectorlayers:
            layernames.append(v[0].name())
            
        notificationDialog = ConfusionMatrixViewer(matrixtext, layernames)
        notificationDialog.setModal(True)
        notificationDialog.show()
        notificationDialog.exec_()
        
        QGisLayers.loadLabelImage(outputclassification, labeldescriptor)
        
        """
        QtGui.QMessageBox.information(  self, \
                                        u"Succès", \
                                        u"La classification a été réalisée avec succès%s%s" % (3*os.linesep, validtext))
        """
        
        return